@@ -372,95 +372,3 @@ totem_interface_get_license (void)
 			  NULL);
 }
 
-/**
- * totem_interface_boldify_label:
- * @builder: a #GtkBuilder
- * @name: the label name
- *
- * Makes the text of the @name label bold.
- *
- * This should be used instead of putting the Pango markup directly into a #GtkBuilder
- * UI file so that translators don't have to translate text formatting markup unnecessarily.
- *
- * This function can only be used if the entire text of a label is to be made bold.
- **/
-void
-totem_interface_boldify_label (GtkBuilder *builder, const char *name)
-{
-	GtkLabel *widget;
-	gchar *str_final;
-
-	widget = GTK_LABEL (gtk_builder_get_object (builder, name));
-
-	if (widget == NULL) {
-		g_warning ("widget '%s' not found", name);
-		return;
-	}
-
-	/* this way is probably better, but for some reason doesn't work with
-	 * labels with mnemonics.
-
-	static PangoAttrList *pattrlist = NULL;
-
-	if (pattrlist == NULL) {
-		PangoAttribute *attr;
-
-		pattrlist = pango_attr_list_new ();
-		attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-		attr->start_index = 0;
-		attr->end_index = G_MAXINT;
-		pango_attr_list_insert (pattrlist, attr);
-	}
-	gtk_label_set_attributes (widget, pattrlist);*/
-
-	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (widget));
-	gtk_label_set_markup_with_mnemonic (widget, str_final);
-	g_free (str_final);
-}
-
-/**
- * totem_interface_italicise_label:
- * @builder: a #GtkBuilder
- * @name: the label name
- *
- * Makes the text of the @name label italic.
- *
- * This should be used instead of putting the Pango markup directly into a #GtkBuilder
- * UI file so that translators don't have to translate text formatting markup unnecessarily.
- *
- * This function can only be used if the entire text of a label is to be made italic.
- **/
-void
-totem_interface_italicise_label (GtkBuilder *builder, const char *name)
-{
-	GtkLabel *widget;
-	gchar *str_final;
-
-	widget = GTK_LABEL (gtk_builder_get_object (builder, name));
-
-	if (widget == NULL) {
-		g_warning ("widget '%s' not found", name);
-		return;
-	}
-
-	/* this way is probably better, but for some reason doesn't work with
-	 * labels with mnemonics.
-
-	static PangoAttrList *pattrlist = NULL;
-
-	if (pattrlist == NULL) {
-		PangoAttribute *attr;
-
-		pattrlist = pango_attr_list_new ();
-		attr = pango_attr_style_new (PANGO_STYLE_ITALIC);
-		attr->start_index = 0;
-		attr->end_index = G_MAXINT;
-		pango_attr_list_insert (pattrlist, attr);
-	}
-	gtk_label_set_attributes (widget, pattrlist);*/
-
-	str_final = g_strdup_printf ("<i>%s</i>", gtk_label_get_label (widget));
-	gtk_label_set_markup_with_mnemonic (widget, str_final);
-	g_free (str_final);
-}
-
