@@ -27,8 +27,6 @@
 
 G_BEGIN_DECLS
 
-#define TOTEM_MAX_RECENT_ITEM_LEN 40
-
 void totem_ui_manager_setup (Totem *totem);
 
 void totem_sublang_update (Totem *totem);
@@ -37,7 +35,9 @@ void totem_sublang_exit (Totem *totem);
 void totem_setup_play_disc (Totem *totem);
 
 void totem_setup_recent (Totem *totem);
-void totem_action_add_recent (Totem *totem, const char *filename);
+void totem_action_add_recent (Totem *totem,
+			      const char *uri,
+			      const char *display_name);
 
 G_END_DECLS
 
