@@ -32,18 +32,17 @@
 #include <gio/gio.h>
 #include <string.h>
 
+#include "eggfileformatchooser.h"
+#include "totem-dnd-menu.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
 #include "video-utils.h"
-#include "debug.h"
 
 #define PL_LEN (gtk_tree_model_iter_n_children (playlist->priv->model, NULL))
 
-static void ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle);
+static void ensure_shuffled (TotemPlaylist *playlist);
 static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl, const char *display_name);
 
-typedef gboolean (*PlaylistCallback) (TotemPlaylist *playlist, const char *mrl,
-		gpointer data);
 typedef gboolean (*ClearComparisonFunc) (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data);
 
 static void totem_playlist_clear_with_compare (TotemPlaylist *playlist,
@@ -51,14 +50,14 @@ static void totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 					       gconstpointer data);
 
 /* Callback function for GtkBuilder */
-void totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist);
-void totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist);
-void playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist);
-void totem_playlist_up_files (GtkWidget *widget, TotemPlaylist *playlist);
-void totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist);
-void playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playlist);
-void playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist);
-void playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist);
+G_MODULE_EXPORT void totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist);
+G_MODULE_EXPORT void totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist);
+G_MODULE_EXPORT void playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist);
+G_MODULE_EXPORT void totem_playlist_up_files (GtkWidget *widget, TotemPlaylist *playlist);
+G_MODULE_EXPORT void totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist);
+G_MODULE_EXPORT void playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playlist);
+G_MODULE_EXPORT void playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist);
+G_MODULE_EXPORT void playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist);
 
 
 typedef struct {
@@ -67,17 +66,6 @@ typedef struct {
 	gpointer user_data;
 } PlaylistForeachContext;
 
-typedef struct {
-	char *mimetype;
-	PlaylistCallback func;
-} PlaylistTypes;
-
-typedef struct {
-	const char *name;
-	const char *suffix;
-	TotemPlParserType type;
-} PlaylistSaveType;
-
 struct TotemPlaylistPrivate
 {
 	GtkBuilder *xml;
@@ -91,12 +79,11 @@ struct TotemPlaylistPrivate
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
 
-	/* This is a scratch list for when we're removing files */
-	GList *list;
-
 	/* These is the current paths for the file selectors */
 	char *path;
 	char *save_path;
+	guint save_format;
+	GtkWidget *file_chooser;
 
 	/* Shuffle mode */
 	int *shuffled;
@@ -108,6 +95,10 @@ struct TotemPlaylistPrivate
 	GtkTreePath *tree_path;
 	GtkTreeViewDropPosition drop_pos;
 
+	/* This is a scratch list for when we're removing files */
+	GList *list;
+	guint current_to_be_removed : 1;
+
 	guint disable_save_to_disk : 1;
 
 	/* Repeat mode */
@@ -147,23 +138,28 @@ enum {
 	NUM_COLS
 };
 
+typedef struct {
+	const char *name;
+	const char *suffix;
+	TotemPlParserType type;
+} PlaylistSaveType;
+
 static PlaylistSaveType save_types [] = {
-	{".PLS", ".pls", TOTEM_PL_PARSER_PLS},
-	{".M3U", ".m3u", TOTEM_PL_PARSER_M3U},
-	{".M3U (DOS)", ".m3u", TOTEM_PL_PARSER_M3U_DOS},
-	{".XSPF", ".xspf", TOTEM_PL_PARSER_XSPF}
+	{ NULL, NULL, -1 }, /* By extension entry */
+	{ N_("MP3 ShoutCast playlist"), "pls", TOTEM_PL_PARSER_PLS },
+	{ N_("MP3 audio (streamed)"), "m3u", TOTEM_PL_PARSER_M3U },
+	{ N_("MP3 audio (streamed, DOS format)"), "m3u", TOTEM_PL_PARSER_M3U_DOS },
+	{ N_("XML Shareable Playlist"), "xspf", TOTEM_PL_PARSER_XSPF }
 };
 
 static int totem_playlist_table_signals[LAST_SIGNAL];
 
+/* casts are to shut gcc up */
 static const GtkTargetEntry target_table[] = {
-	{ "text/uri-list", 0, 0 },
-	{ "_NETSCAPE_URL", 0, 1 }
+	{ (gchar*) "text/uri-list", 0, 0 },
+	{ (gchar*) "_NETSCAPE_URL", 0, 1 }
 };
 
-static void totem_playlist_class_init (TotemPlaylistClass *class);
-static void totem_playlist_init       (TotemPlaylist      *playlist);
-
 static void init_treeview (GtkWidget *treeview, TotemPlaylist *playlist);
 
 #define totem_playlist_unset_playing(x) totem_playlist_set_playing(x, TOTEM_PLAYLIST_STATUS_NONE)
@@ -246,9 +242,9 @@ totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				title);
+				"%s", title);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
-						  reason);
+						  "%s", reason);
 
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
@@ -262,6 +258,88 @@ totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 	gtk_widget_show (error_dialog);
 }
 
+void
+totem_playlist_select_subtitle_dialog(TotemPlaylist *playlist, TotemPlaylistSelectDialog mode)
+{
+	char *subtitle, *current, *uri;
+	GFile *file, *dir;
+	TotemPlaylistStatus playing;
+	GtkTreeIter iter;
+
+	if (mode == TOTEM_PLAYLIST_DIALOG_PLAYING) {
+		/* Set subtitle file for the currently playing movie */
+		gtk_tree_model_get_iter (playlist->priv->model, &iter, playlist->priv->current);
+	} else if (mode == TOTEM_PLAYLIST_DIALOG_SELECTED) {
+		/* Set subtitle file in for the first selected playlist item */
+		GList *l;
+
+		l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
+		gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
+		g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (l);
+	} else {
+		g_assert_not_reached ();
+	}
+
+	/* Look for the directory of the current movie */
+	gtk_tree_model_get (playlist->priv->model, &iter,
+			    URI_COL, &current,
+			    -1);
+
+	if (current == NULL)
+		return;
+
+	uri = NULL;
+	file = g_file_new_for_uri (current);
+	dir = g_file_get_parent (file);
+	g_object_unref (file);
+	if (dir != NULL) {
+		uri = g_file_get_uri (dir);
+		g_object_unref (dir);
+	}
+
+	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), uri);
+	g_free (uri);
+
+	if (subtitle == NULL)
+		return;
+
+	gtk_tree_model_get (playlist->priv->model, &iter,
+			    PLAYING_COL, &playing,
+			    -1);
+
+	gtk_list_store_set (GTK_LIST_STORE(playlist->priv->model), &iter,
+			    SUBTITLE_URI_COL, subtitle,
+			    -1);
+
+	if (playing != TOTEM_PLAYLIST_STATUS_NONE) {
+		g_signal_emit (G_OBJECT (playlist),
+			       totem_playlist_table_signals[SUBTITLE_CHANGED], 0,
+			       NULL);
+	}
+
+	g_free(subtitle);
+}
+
+void
+totem_playlist_set_current_subtitle (TotemPlaylist *playlist, const char *subtitle_uri)
+{
+	GtkTreeIter iter;
+
+	if (playlist->priv->current == NULL)
+		return;
+
+	gtk_tree_model_get_iter (playlist->priv->model, &iter, playlist->priv->current);
+
+	gtk_list_store_set (GTK_LIST_STORE(playlist->priv->model), &iter,
+			    SUBTITLE_URI_COL, subtitle_uri,
+			    -1);
+
+	g_signal_emit (G_OBJECT (playlist),
+		       totem_playlist_table_signals[SUBTITLE_CHANGED], 0,
+		       NULL);
+}
+
 /* This one returns a new string, in UTF8 even if the MRL is encoded
  * in the locale's encoding
  */
@@ -384,13 +462,24 @@ drop_cb (GtkWidget        *widget,
 	 gint              y,
 	 GtkSelectionData *data, 
 	 guint             info, 
-	 guint             time, 
+	 guint             _time, 
 	 TotemPlaylist    *playlist)
 {
 	char **list;
 	GList *p, *file_list;
 	guint i;
 
+	if (context->suggested_action == GDK_ACTION_ASK) {
+		context->action = totem_drag_ask (PL_LEN != 0);
+		if (context->action == GDK_ACTION_DEFAULT) {
+			gtk_drag_finish (context, FALSE, FALSE, _time);
+			return;
+		}
+	}
+
+	if (context->action == GDK_ACTION_MOVE)
+		totem_playlist_clear (playlist);
+
 	list = g_uri_list_extract_uris ((char *)data->data);
 	file_list = NULL;
 
@@ -402,7 +491,7 @@ drop_cb (GtkWidget        *widget,
 	}
 
 	if (file_list == NULL) {
-		gtk_drag_finish (context, FALSE, FALSE, time);
+		gtk_drag_finish (context, FALSE, FALSE, _time);
 		return;
 	}
 
@@ -447,7 +536,7 @@ drop_cb (GtkWidget        *widget,
 
 	g_strfreev (list);
 	g_list_free (file_list);
-	gtk_drag_finish (context, TRUE, FALSE, time);
+	gtk_drag_finish (context, TRUE, FALSE, _time);
 	gtk_tree_path_free (playlist->priv->tree_path);
 	playlist->priv->tree_path = NULL;
 
@@ -461,55 +550,7 @@ drop_cb (GtkWidget        *widget,
 void
 playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
-	char *subtitle, *current, *path;
-	GList *l;
-	GFile *file, *dir;
-	TotemPlaylistStatus playing;
-	GtkTreeIter iter;
-
-	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
-	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
-	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
-	g_list_free (l);
-
-	/* Look for the directory of the current movie */
-	gtk_tree_model_get (playlist->priv->model, &iter,
-			    FILENAME_COL, &current,
-			    -1);
-
-	if (current == NULL)
-		return;
-
-	path = NULL;
-	file = g_file_new_for_commandline_arg (current);
-	dir = g_file_get_parent (file);
-	g_object_unref (file);
-	if (dir != NULL) {
-		path = g_file_get_path (dir);
-		g_object_unref (dir);
-	}
-
-	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), path);
-	g_free (path);
-
-	if (subtitle == NULL)
-		return;
-
-	gtk_tree_model_get (playlist->priv->model, &iter,
-			    PLAYING_COL, &playing,
-			    -1);
-
-	gtk_list_store_set (GTK_LIST_STORE(playlist->priv->model), &iter, 
-			    SUBTITLE_URI_COL, subtitle,
-			    -1);
-
-	if (playing != TOTEM_PLAYLIST_STATUS_NONE) {
-		g_signal_emit (G_OBJECT (playlist),
-			       totem_playlist_table_signals[SUBTITLE_CHANGED], 0,
-			       NULL);
-	}
-
-	g_free(subtitle);
+	totem_playlist_select_subtitle_dialog (playlist, TOTEM_PLAYLIST_DIALOG_SELECTED);
 }
 
 void
@@ -546,7 +587,7 @@ static gboolean
 playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 {
 	guint button = 0;
-       	guint32 time;
+	guint32 _time;
 	GtkTreePath *path;
 	gint count;
 	GtkWidget *menu;
@@ -555,7 +596,7 @@ playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 
 	if (event != NULL) {
 		button = event->button;
-		time = event->time;
+		_time = event->time;
 
 		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (playlist->priv->treeview),
 				 event->x, event->y, &path, NULL, NULL, NULL)) {
@@ -568,7 +609,7 @@ playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 			gtk_tree_selection_unselect_all (playlist->priv->selection);
 		}
 	} else {
-		time = gtk_get_current_event_time ();
+		_time = gtk_get_current_event_time ();
 	}
 
 	count = gtk_tree_selection_count_selected_rows (playlist->priv->selection);
@@ -587,7 +628,7 @@ playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			button, time);
+			button, _time);
 
 	return TRUE;
 }
@@ -624,19 +665,19 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 	for (i = 0; i < num_items; i++)
 	{
 		GtkTreeIter iter;
-		char *index;
+		char *playlist_index;
 		GtkTreePath *path;
 		TotemPlaylistStatus playing;
 
-		index = g_strdup_printf ("%d", i);
+		playlist_index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
 				(playlist->priv->model,
-				 &iter, index) == FALSE)
+				 &iter, playlist_index) == FALSE)
 		{
-			g_free (index);
+			g_free (playlist_index);
 			continue;
 		}
-		g_free (index);
+		g_free (playlist_index);
 
 		gtk_tree_model_get (playlist->priv->model, &iter, PLAYING_COL, &playing, -1);
 		if (playing == TOTEM_PLAYLIST_STATUS_NONE)
@@ -644,8 +685,7 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 
 		/* Only emit the changed signal if we changed the ->current */
 		path = gtk_tree_path_new_from_indices (i, -1);
-		if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
-		{
+		if (gtk_tree_path_compare (path, playlist->priv->current) == 0) {
 			gtk_tree_path_free (path);
 		} else {
 			gtk_tree_path_free (playlist->priv->current);
@@ -812,6 +852,10 @@ totem_playlist_foreach_selected (GtkTreeModel *model, GtkTreePath *path,
 	ref = gtk_tree_row_reference_new (playlist->priv->model, path);
 	playlist->priv->list = g_list_prepend
 		(playlist->priv->list, (gpointer) ref);
+	if (playlist->priv->current_to_be_removed == FALSE
+	    && playlist->priv->current != NULL
+	    && gtk_tree_path_compare (path, playlist->priv->current) == 0)
+		playlist->priv->current_to_be_removed = TRUE;
 }
 
 static void
@@ -835,110 +879,7 @@ totem_playlist_emit_item_removed (TotemPlaylist *playlist,
 static void
 playlist_remove_files (TotemPlaylist *playlist)
 {
-	GtkTreeSelection *selection;
-	GtkTreeRowReference *ref;
-	gboolean is_selected = FALSE;
-	int next_pos;
-
-	selection = gtk_tree_view_get_selection
-		(GTK_TREE_VIEW (playlist->priv->treeview));
-	if (selection == NULL)
-		return;
-
-	gtk_tree_selection_selected_foreach (selection,
-			totem_playlist_foreach_selected,
-			(gpointer) playlist);
-
-	/* If the current item is to change, we need to keep an static
-	 * reference to it, TreeIter and TreePath don't allow that */
-	if (playlist->priv->current != NULL)
-	{
-		int *indices;
-
-		ref = gtk_tree_row_reference_new (playlist->priv->model,
-				playlist->priv->current);
-		is_selected = gtk_tree_selection_path_is_selected (selection,
-				playlist->priv->current);
-
-		indices = gtk_tree_path_get_indices (playlist->priv->current);
-		next_pos = indices[0];
-
-		gtk_tree_path_free (playlist->priv->current);
-	} else {
-		ref = NULL;
-		next_pos = -1;
-	}
-
-	/* We destroy the items, one-by-one from the list built above */
-	while (playlist->priv->list != NULL)
-	{
-		GtkTreePath *path;
-		GtkTreeIter iter;
-
-		path = gtk_tree_row_reference_get_path
-			((GtkTreeRowReference *)(playlist->priv->list->data));
-		gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
-		gtk_tree_path_free (path);
-
-		totem_playlist_emit_item_removed (playlist, &iter);
-		gtk_list_store_remove (GTK_LIST_STORE (playlist->priv->model), &iter);
-
-		gtk_tree_row_reference_free
-			((GtkTreeRowReference *)(playlist->priv->list->data));
-		playlist->priv->list = g_list_remove (playlist->priv->list,
-				playlist->priv->list->data);
-	}
-	g_list_free (playlist->priv->list);
-	playlist->priv->list = NULL;
-
-	if (is_selected != FALSE)
-	{
-		/* The current item was removed from the playlist */
-		if (next_pos != -1)
-		{
-			char *str;
-			GtkTreeIter iter;
-			GtkTreePath *cur;
-
-			str = g_strdup_printf ("%d", next_pos);
-			cur = gtk_tree_path_new_from_string (str);
-
-			if (gtk_tree_model_get_iter (playlist->priv->model,
-						&iter, cur) == FALSE)
-			{
-				playlist->priv->current = NULL;
-				gtk_tree_path_free (cur);
-			} else {
-				playlist->priv->current = cur;
-			}
-			g_free (str);
-		} else {
-			playlist->priv->current = NULL;
-		}
-
-		playlist->priv->current_shuffled = -1;
-		ensure_shuffled (playlist, playlist->priv->shuffle);
-
-		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CURRENT_REMOVED],
-				0, NULL);
-	} else {
-		if (ref != NULL)
-		{
-			/* The path to the current item changed */
-			playlist->priv->current =
-				gtk_tree_row_reference_get_path (ref);
-			gtk_tree_row_reference_free (ref);
-		}
-
-		ensure_shuffled (playlist, playlist->priv->shuffle);
-
-		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CHANGED], 0,
-				NULL);
-	}
-	totem_playlist_update_save_button (playlist);
-	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->priv->treeview));
+	totem_playlist_clear_with_compare (playlist, NULL, NULL);
 }
 
 void
@@ -956,103 +897,144 @@ playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist)
 static void
 totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint active_format)
 {
-	PlaylistSaveType *cur = NULL;
-	guint i;
+	if (active_format == 0)
+		active_format = 1;
 
-	if (active_format > 0)
-		totem_playlist_save_current_playlist_ext (playlist, filename,
-							save_types[active_format - 1].type);
-	else {
-		for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-			if (g_str_has_suffix (filename, save_types[i].suffix)) {
-				cur = &save_types[i];
-				break;
+	totem_playlist_save_current_playlist_ext (playlist, filename,
+						  save_types[active_format].type);
+}
+
+static char *
+suffix_match_replace (const char *fname, guint old_format, guint new_format)
+{
+	char *ext;
+
+	ext = g_strdup_printf (".%s", save_types[old_format].suffix);
+	if (g_str_has_suffix (fname, ext) != FALSE) {
+		char *no_suffix, *new_fname;
+
+		no_suffix = g_strndup (fname, strlen (fname) - strlen (ext));
+		new_fname = g_strconcat (no_suffix, ".", save_types[new_format].suffix, NULL);
+		g_free (no_suffix);
+		g_free (ext);
+
+		return new_fname;
+	}
+	g_free (ext);
+
+	return NULL;
+}
+
+static void
+format_selection_changed (EggFileFormatChooser *chooser, TotemPlaylist *playlist)
+{
+	guint format;
+
+	format = egg_file_format_chooser_get_format (chooser, NULL);
+
+	if (format != playlist->priv->save_format) {
+		char *fname, *new_fname;
+
+		new_fname = NULL;
+		fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (playlist->priv->file_chooser));
+
+		if (format == 0) {
+			/* The new format is "By extension" don't touch anything */
+		} else if (playlist->priv->save_format == 0) {
+			guint i;
+
+			for (i = 1; i < G_N_ELEMENTS (save_types); i++) {
+				new_fname = suffix_match_replace (fname, i, format);
+				if (new_fname != NULL)
+					break;
 			}
+		} else {
+			new_fname = suffix_match_replace (fname, playlist->priv->save_format, format);
 		}
-		if (cur == NULL)
-			totem_playlist_error (_("Could not save the playlist"), _("Unknown file extension."), playlist);
-		else
-			totem_playlist_save_current_playlist_ext (playlist, filename, cur->type);
+		if (new_fname != NULL) {
+			char *basename;
+
+			basename = g_path_get_basename (new_fname);
+			g_free (new_fname);
+			gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (playlist->priv->file_chooser), basename);
+			g_free (basename);
+		}
+		playlist->priv->save_format = format;
 	}
 }
 
 static GtkWidget *
-totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
+totem_playlist_save_add_format_chooser (GtkFileChooser *fc, TotemPlaylist *playlist)
 {
-	GtkWidget *hbox, *label, *combo_box;
+	GtkWidget *format_chooser;
 	guint i;
 
-	hbox = gtk_hbox_new (FALSE, 4);
-	label = gtk_label_new (_("Select playlist format:"));
-	gtk_widget_show (label);
+	format_chooser = egg_file_format_chooser_new ();
 
-	combo_box = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
-			_("By extension"));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
-	for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
-				save_types[i].name);
+	playlist->priv->save_format = 0;
+
+	for (i = 1; i < G_N_ELEMENTS (save_types) ; i++) {
+		egg_file_format_chooser_add_format (
+		    EGG_FILE_FORMAT_CHOOSER (format_chooser), 0, _(save_types[i].name),
+		    "gnome-mime-audio", save_types[i].suffix, NULL);
 	}
-	gtk_widget_show (combo_box);
 
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
-	gtk_widget_show (hbox);
-	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc), hbox);
+	g_signal_connect (format_chooser, "selection-changed",
+			  G_CALLBACK (format_selection_changed), playlist);
 
-	atk_object_add_relationship (gtk_widget_get_accessible (label),
-			ATK_RELATION_LABEL_FOR,
-			gtk_widget_get_accessible (combo_box));
-	atk_object_add_relationship (gtk_widget_get_accessible (combo_box),
-			ATK_RELATION_LABELLED_BY,
-			gtk_widget_get_accessible (label));
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc),
+					   format_chooser);
 
-	return combo_box;
+	return format_chooser;
 }
 
 void
 totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
-	GtkWidget *fs, *combo_box;
+	GtkWidget *fs, *format_chooser;
 	char *filename;
 	int response;
 
+	g_assert (playlist->priv->file_chooser == NULL);
+
 	fs = gtk_file_chooser_dialog_new (_("Save Playlist"),
-			totem_playlist_get_toplevel (playlist),
-			GTK_FILE_CHOOSER_ACTION_SAVE,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-			NULL);
+					  totem_playlist_get_toplevel (playlist),
+					  GTK_FILE_CHOOSER_ACTION_SAVE,
+					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					  NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
 	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (fs), TRUE);
+
 	/* translators: Playlist is the default saved playlist filename,
 	 * without the suffix */
-	filename = g_strconcat (_("Playlist"), save_types[0].suffix, NULL);
+	filename = g_strconcat (_("Playlist"), ".", save_types[1].suffix, NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (fs), filename);
 	g_free (filename);
-	combo_box = totem_playlist_save_add_format_combo_box (GTK_FILE_CHOOSER (fs));
+	format_chooser = totem_playlist_save_add_format_chooser (GTK_FILE_CHOOSER (fs), playlist);
 
-	if (playlist->priv->save_path != NULL)
-	{
+	if (playlist->priv->save_path != NULL) {
 		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
 				playlist->priv->save_path);
 	}
 
+	playlist->priv->file_chooser = fs;
+
 	response = gtk_dialog_run (GTK_DIALOG (fs));
 	gtk_widget_hide (fs);
 	while (gtk_events_pending())
 		gtk_main_iteration();
 
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
+	if (response == GTK_RESPONSE_ACCEPT) {
 		char *fname;
-		gint active_format;
+		guint active_format;
 
 		fname = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
-		active_format = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+		active_format = egg_file_format_chooser_get_format (EGG_FILE_FORMAT_CHOOSER (format_chooser),
+								    fname);
 
+		playlist->priv->file_chooser = NULL;
 		gtk_widget_destroy (fs);
 
 		if (fname == NULL)
@@ -1064,6 +1046,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 		totem_playlist_save_playlist (playlist, fname, active_format);
 		g_free (fname);
 	} else {
+		playlist->priv->file_chooser = NULL;
 		gtk_widget_destroy (fs);
 	}
 }
@@ -1090,8 +1073,7 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	pos = -2;
 	refs = NULL;
 
-	if (playlist->priv->current != NULL)
-	{
+	if (playlist->priv->current != NULL) {
 		current = gtk_tree_row_reference_new (model,
 				playlist->priv->current);
 	} else {
@@ -1100,8 +1082,7 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 
 	/* Build a list of tree references */
 	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
-	for (l = paths; l != NULL; l = l->next)
-	{
+	for (l = paths; l != NULL; l = l->next) {
 		GtkTreePath *path = l->data;
 		int cur_pos, *indices;
 
@@ -1131,14 +1112,12 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	else
 		pos = pos - 2;
 
-	for (l = refs; l != NULL; l = l->next)
-	{
+	for (l = refs; l != NULL; l = l->next) {
 		GtkTreeIter *position, cur;
 		GtkTreeRowReference *ref = l->data;
 		GtkTreePath *path;
 
-		if (pos < 0)
-		{
+		if (pos < 0) {
 			position = NULL;
 		} else {
 			char *str;
@@ -1171,8 +1150,7 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	g_list_free (refs);
 
 	/* Update the current path */
-	if (current != NULL)
-	{
+	if (current != NULL) {
 		gtk_tree_path_free (playlist->priv->current);
 		playlist->priv->current = gtk_tree_row_reference_get_path
 			(current);
@@ -1401,38 +1379,34 @@ compare_random (gconstpointer ptr_a, gconstpointer ptr_b)
 }
 
 static void
-ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle)
+ensure_shuffled (TotemPlaylist *playlist)
 {
 	RandomData data;
 	GArray *array;
-	int i, current;
+	int i, current, current_new;
 	int *indices;
 
-	if (shuffle == FALSE || PL_LEN != playlist->priv->shuffle_len)
-	{
-		g_free (playlist->priv->shuffled);
-		playlist->priv->shuffled = NULL;
-	}
+	if (playlist->priv->shuffled == NULL)
+		playlist->priv->shuffled = g_new (int, PL_LEN);
+	else if (PL_LEN != playlist->priv->shuffle_len)
+		playlist->priv->shuffled = g_renew (int, playlist->priv->shuffled, PL_LEN);
+	playlist->priv->shuffle_len = PL_LEN;
 
-	if (shuffle == FALSE || PL_LEN == 0)
+	if (PL_LEN == 0)
 		return;
 
-	if (playlist->priv->current != NULL)
-	{
+	if (playlist->priv->current != NULL) {
 		indices = gtk_tree_path_get_indices (playlist->priv->current);
 		current = indices[0];
 	} else {
 		current = -1;
 	}
+	
+	current_new = -1;
 
-	playlist->priv->shuffled = g_new (int, PL_LEN);
-	playlist->priv->shuffle_len = PL_LEN;
+	array = g_array_sized_new (FALSE, FALSE, sizeof (RandomData), PL_LEN);
 
-	array = g_array_sized_new (FALSE, FALSE,
-			sizeof (RandomData), PL_LEN);
-
-	for (i = 0; i < PL_LEN; i++)
-	{
+	for (i = 0; i < PL_LEN; i++) {
 		data.random = g_random_int_range (0, PL_LEN);
 		data.index = i;
 
@@ -1441,14 +1415,17 @@ ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle)
 
 	g_array_sort (array, compare_random);
 
-	for (i = 0; i < PL_LEN; i++)
-	{
-		playlist->priv->shuffled[i]
-			= g_array_index (array, RandomData, i).index;
+	for (i = 0; i < PL_LEN; i++) {
+		playlist->priv->shuffled[i] = g_array_index (array, RandomData, i).index;
 
-		if (playlist->priv->current != NULL
-				&& playlist->priv->shuffled[i] == current)
-			playlist->priv->current_shuffled = i;
+		if (playlist->priv->current != NULL && playlist->priv->shuffled[i] == current)
+			current_new = i;
+	}
+
+	if (current_new > -1) {
+		playlist->priv->shuffled[current_new] = playlist->priv->shuffled[0];
+		playlist->priv->shuffled[0] = current;
+		playlist->priv->current_shuffled = 0;
 	}
 
 	g_array_free (array, TRUE);
@@ -1462,7 +1439,13 @@ update_shuffle_cb (GConfClient *client, guint cnxn_id,
 
 	shuffle = gconf_value_get_bool (entry->value);
 	playlist->priv->shuffle = shuffle;
-	ensure_shuffled (playlist, shuffle);
+	if (shuffle == FALSE) {
+		g_free (playlist->priv->shuffled);
+		playlist->priv->shuffled = NULL;
+		playlist->priv->shuffle_len = 0;
+	} else {
+		ensure_shuffled (playlist);
+	}
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
@@ -1532,6 +1515,40 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
 
+static gboolean
+totem_playlist_compare_with_monitor (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data)
+{
+	GFileMonitor *monitor = (GFileMonitor *) data;
+	GFileMonitor *_monitor;
+	gboolean retval = FALSE;
+
+	gtk_tree_model_get (playlist->priv->model, iter,
+			    FILE_MONITOR_COL, &_monitor, -1);
+
+	if (_monitor == monitor)
+		retval = TRUE;
+
+	if (_monitor != NULL)
+		g_object_unref (_monitor);
+
+	return retval;
+}
+
+static void
+totem_playlist_file_changed (GFileMonitor *monitor,
+			     GFile *file,
+			     GFile *other_file,
+			     GFileMonitorEvent event_type,
+			     TotemPlaylist *playlist)
+{
+	if (event_type == G_FILE_MONITOR_EVENT_PRE_UNMOUNT ||
+	    event_type == G_FILE_MONITOR_EVENT_UNMOUNTED) {
+		totem_playlist_clear_with_compare (playlist,
+						   (ClearComparisonFunc) totem_playlist_compare_with_monitor,
+						   monitor);
+	}
+}
+
 static void
 totem_playlist_dispose (GObject *object)
 {
@@ -1591,10 +1608,10 @@ totem_playlist_init (TotemPlaylist *playlist)
 	/* popup menu */
 	playlist->priv->action_group = GTK_ACTION_GROUP (gtk_builder_get_object (playlist->priv->xml, "playlist-action-group"));
 	playlist->priv->ui_manager = GTK_UI_MANAGER (gtk_builder_get_object (playlist->priv->xml, "totem-playlist-ui-manager"));
-
+	
 	gtk_widget_add_events (GTK_WIDGET (playlist), GDK_KEY_PRESS_MASK);
 	g_signal_connect (G_OBJECT (playlist), "key_press_event",
-			G_CALLBACK (totem_playlist_key_press), playlist);
+			  G_CALLBACK (totem_playlist_key_press), playlist);
 
 	/* Reparent the vbox */
 	container = GTK_WIDGET (gtk_builder_get_object (playlist->priv->xml, "vbox4"));
@@ -1612,6 +1629,10 @@ totem_playlist_init (TotemPlaylist *playlist)
 	playlist->priv->model = gtk_tree_view_get_model
 		(GTK_TREE_VIEW (playlist->priv->treeview));
 
+	/* tooltips */
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(playlist->priv->treeview),
+					 FILENAME_COL);
+
 	/* The configuration */
 	init_config (playlist);
 
@@ -1640,12 +1661,14 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	GtkTreeIter iter;
 	char *filename_for_display, *uri;
 	GtkTreeRowReference *ref;
+	GFileMonitor *monitor;
+	GFile *file;
 	int pos;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
-	if (display_name == NULL)
+	if (display_name == NULL || *display_name == '\0')
 		filename_for_display = totem_playlist_mrl_to_title (mrl);
 	else
 		filename_for_display = g_strdup (display_name);
@@ -1653,8 +1676,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	ref = NULL;
 	uri = totem_create_full_path (mrl);
 
-	D("totem_playlist_add_one_mrl (): %s %s %s\n",
-				filename_for_display, uri ? uri : "(null)", display_name);
+	g_debug ("totem_playlist_add_one_mrl (): %s %s %s\n", filename_for_display, uri, display_name);
 
 	if (playlist->priv->tree_path != NULL && playlist->priv->current != NULL) {
 		int *indices;
@@ -1667,11 +1689,27 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 
 	store = GTK_LIST_STORE (playlist->priv->model);
 
+	/* Get the file monitor */
+	file = g_file_new_for_uri (uri ? uri : mrl);
+	if (g_file_is_native (file) != FALSE) {
+		monitor = g_file_monitor_file (file,
+					       G_FILE_MONITOR_NONE,
+					       NULL,
+					       NULL);
+		g_signal_connect (G_OBJECT (monitor),
+				  "changed",
+				  G_CALLBACK (totem_playlist_file_changed),
+				  playlist);
+	} else {
+		monitor = NULL;
+	}
+
 	gtk_list_store_insert_with_values (store, &iter, pos,
 			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri ? uri : mrl,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
+			FILE_MONITOR_COL, monitor,
 			-1);
 
 	g_signal_emit (playlist,
@@ -1683,7 +1721,8 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 
 	if (playlist->priv->current == NULL && playlist->priv->shuffle == FALSE)
 		playlist->priv->current = gtk_tree_model_get_path (playlist->priv->model, &iter);
-	ensure_shuffled (playlist, playlist->priv->shuffle);
+	if (playlist->priv->shuffle)
+		ensure_shuffled (playlist);
 
 	/* And update current to point to the right file again */
 	if (ref != NULL) {
@@ -1781,78 +1820,141 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 				   ClearComparisonFunc func,
 				   gconstpointer data)
 {
-	GList *list = NULL, *l;
-	guint num_items, i;
-	gboolean has_items, current_removed;
+	GtkTreeRowReference *ref;
+	int next_pos;
 
-	num_items = PL_LEN;
-	if (num_items == 0)
-		return;
+	ref = NULL;
+	next_pos = -1;
 
-	current_removed = FALSE;
+	if (func == NULL) {
+		GtkTreeSelection *selection;
 
-	for (i = 0; i < num_items; i++)
-	{
-		GtkTreeIter iter;
-		char *index;
+		selection = gtk_tree_view_get_selection
+			(GTK_TREE_VIEW (playlist->priv->treeview));
+		if (selection == NULL)
+			return;
 
-		index = g_strdup_printf ("%d", i);
-		if (gtk_tree_model_get_iter_from_string
-				(playlist->priv->model,
-				 &iter, index) == FALSE)
-		{
-			g_free (index);
-			continue;
+		gtk_tree_selection_selected_foreach (selection,
+						     totem_playlist_foreach_selected,
+						     (gpointer) playlist);
+	} else {
+		guint num_items, i;
+
+		num_items = PL_LEN;
+		if (num_items == 0)
+			return;
+
+		for (i = 0; i < num_items; i++) {
+			GtkTreeIter iter;
+			char *playlist_index;
+
+			playlist_index = g_strdup_printf ("%d", i);
+			if (gtk_tree_model_get_iter_from_string (playlist->priv->model, &iter, playlist_index) == FALSE) {
+				g_free (playlist_index);
+				continue;
+			}
+			g_free (playlist_index);
+
+			if ((* func) (playlist, &iter, data) != FALSE) {
+				GtkTreePath *path;
+				GtkTreeRowReference *r;
+
+				path = gtk_tree_path_new_from_indices (i, -1);
+				r = gtk_tree_row_reference_new (playlist->priv->model, path);
+				if (playlist->priv->current != NULL) {
+					if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
+						playlist->priv->current_to_be_removed = TRUE;
+				}
+				playlist->priv->list = g_list_prepend (playlist->priv->list, r);
+				gtk_tree_path_free (path);
+			}
 		}
-		g_free (index);
 
-		if ((* func) (playlist, &iter, data) != FALSE)
-		{
-			GtkTreePath *path;
-			GtkTreeRowReference *ref;
-
-			path = gtk_tree_path_new_from_indices (i, -1);
-			ref = gtk_tree_row_reference_new
-				(playlist->priv->model, path);
-			list = g_list_prepend (list, ref);
-			gtk_tree_path_free (path);
-		}
+		if (playlist->priv->list == NULL)
+			return;
 	}
 
-	has_items = (list != NULL);
+	/* If the current item is to change, we need to keep an static
+	 * reference to it, TreeIter and TreePath don't allow that */
+	if (playlist->priv->current != NULL) {
+		int *indices;
 
-	for (l = list; l != NULL; l = l->next)
+		ref = gtk_tree_row_reference_new (playlist->priv->model,
+				playlist->priv->current);
+		indices = gtk_tree_path_get_indices (playlist->priv->current);
+		next_pos = indices[0];
+
+	}
+
+	/* We destroy the items, one-by-one from the list built above */
+	while (playlist->priv->list != NULL)
 	{
 		GtkTreePath *path;
 		GtkTreeIter iter;
 
-		path = gtk_tree_row_reference_get_path (l->data);
+		path = gtk_tree_row_reference_get_path
+			((GtkTreeRowReference *)(playlist->priv->list->data));
 		gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
-		if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
-			current_removed = TRUE;
+		gtk_tree_path_free (path);
 
 		totem_playlist_emit_item_removed (playlist, &iter);
 		gtk_list_store_remove (GTK_LIST_STORE (playlist->priv->model), &iter);
-		gtk_tree_path_free (path);
-		gtk_tree_row_reference_free (l->data);
+
+		gtk_tree_row_reference_free
+			((GtkTreeRowReference *)(playlist->priv->list->data));
+		playlist->priv->list = g_list_remove (playlist->priv->list,
+				playlist->priv->list->data);
 	}
-	g_list_free (list);
+	g_list_free (playlist->priv->list);
+	playlist->priv->list = NULL;
+
+	if (playlist->priv->current_to_be_removed != FALSE) {
+		/* The current item was removed from the playlist */
+		if (next_pos != -1) {
+			char *str;
+			GtkTreeIter iter;
+			GtkTreePath *cur;
+
+			str = g_strdup_printf ("%d", next_pos);
+			cur = gtk_tree_path_new_from_string (str);
+
+			if (gtk_tree_model_get_iter (playlist->priv->model, &iter, cur) == FALSE) {
+				playlist->priv->current = NULL;
+				gtk_tree_path_free (cur);
+			} else {
+				playlist->priv->current = cur;
+			}
+			g_free (str);
+		} else {
+			playlist->priv->current = NULL;
+		}
 
-	if (has_items != FALSE && current_removed != FALSE) {
 		playlist->priv->current_shuffled = -1;
+		if (playlist->priv->shuffle)
+			ensure_shuffled (playlist);
 
-		ensure_shuffled (playlist, playlist->priv->shuffle);
-		gtk_tree_path_free (playlist->priv->current);
-		playlist->priv->current = NULL;
 		g_signal_emit (G_OBJECT (playlist),
 				totem_playlist_table_signals[CURRENT_REMOVED],
 				0, NULL);
-	} else if (has_items != FALSE) {
-		ensure_shuffled (playlist, playlist->priv->shuffle);
+	} else {
+		if (ref != NULL) {
+			/* The path to the current item changed */
+			playlist->priv->current =
+				gtk_tree_row_reference_get_path (ref);
+			gtk_tree_row_reference_free (ref);
+		}
+
+		if (playlist->priv->shuffle)
+			ensure_shuffled (playlist);
+
 		g_signal_emit (G_OBJECT (playlist),
-			       totem_playlist_table_signals[CHANGED], 0,
-			       NULL);
+				totem_playlist_table_signals[CHANGED], 0,
+				NULL);
 	}
+	totem_playlist_update_save_button (playlist);
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->priv->treeview));
+
+	playlist->priv->current_to_be_removed = FALSE;
 }
 
 static gboolean
@@ -1934,17 +2036,24 @@ totem_playlist_get_current_title (TotemPlaylist *playlist, gboolean *custom)
 			&iter,
 			playlist->priv->current);
 
-	gtk_tree_model_get (playlist->priv->model,
-			&iter,
-			FILENAME_COL, &path,
-			TITLE_CUSTOM_COL, custom,
-			-1);
+	if (custom != NULL) {
+		gtk_tree_model_get (playlist->priv->model,
+				    &iter,
+				    FILENAME_COL, &path,
+				    TITLE_CUSTOM_COL, custom,
+				    -1);
+	} else {
+		gtk_tree_model_get (playlist->priv->model,
+				    &iter,
+				    FILENAME_COL, &path,
+				    -1);
+	}
 
 	return path;
 }
 
 char *
-totem_playlist_get_title (TotemPlaylist *playlist, guint index)
+totem_playlist_get_title (TotemPlaylist *playlist, guint title_index)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
@@ -1952,7 +2061,7 @@ totem_playlist_get_title (TotemPlaylist *playlist, guint index)
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
 
-	path = gtk_tree_path_new_from_indices (index, -1);
+	path = gtk_tree_path_new_from_indices (title_index, -1);
 
 	gtk_tree_model_get_iter (playlist->priv->model,
 				 &iter, path);
@@ -2281,46 +2390,51 @@ totem_playlist_set_at_end (TotemPlaylist *playlist)
 	}
 }
 
-guint
+int
 totem_playlist_get_current (TotemPlaylist *playlist)
 {
 	char *path;
-	double index;
+	double current_index;
 
-	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), 0);
-
-	if (playlist->priv->current == NULL)
-		return 0;
-	path = gtk_tree_path_to_string (playlist->priv->current);
-	if (path == NULL)
-		return 0;
-
-	index = g_ascii_strtod (path, NULL);
-	g_free (path);
-
-	return index;
-}
-
-guint
-totem_playlist_get_last (TotemPlaylist *playlist)
-{
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), -1);
 
-	return PL_LEN - 1;
+	if (playlist->priv->current == NULL)
+		return -1;
+	path = gtk_tree_path_to_string (playlist->priv->current);
+	if (path == NULL)
+		return -1;
+
+	current_index = g_ascii_strtod (path, NULL);
+	g_free (path);
+
+	return current_index;
+}
+
+int
+totem_playlist_get_last (TotemPlaylist *playlist)
+{
+	guint len = PL_LEN;
+
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), -1);
+
+	if (len == 0)
+		return -1;
+
+	return len - 1;
 }
 
 void
-totem_playlist_set_current (TotemPlaylist *playlist, guint index)
+totem_playlist_set_current (TotemPlaylist *playlist, guint current_index)
 {
 	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
-	if (index >= (guint) PL_LEN)
+	if (current_index >= (guint) PL_LEN)
 		return;
 
 	totem_playlist_unset_playing (playlist);
 	//FIXME problems when shuffled?
 	gtk_tree_path_free (playlist->priv->current);
-	playlist->priv->current = gtk_tree_path_new_from_indices (index, -1);
+	playlist->priv->current = gtk_tree_path_new_from_indices (current_index, -1);
 }
 
 static void
